# -*- coding: utf-8 -*-
# AUTHOR Dries Vervoort (Luxion)
# VERSION 0.0.1
# Batch render Toy Train models to images.
import os

def getGeometryImportOptions():
  importOptions = lux.getImportOptions()
  importOptions['center_geometry'] = False
  importOptions['snap_to_ground'] = True
  importOptions['up_vector'] = 2
  importOptions['adjust_camera_look_at'] = False
  importOptions['adjust_environment'] = False
  importOptions['update_mode'] = False
  importOptions['group_by_shader'] = False
  importOptions['group_by'] = 2
  importOptions['new_import'] = False
  importOptions['tessellation_quality'] = 0.15
  importOptions['accurate_tessellation'] = True
  return importOptions

def setRenderOptions(samples, lightingPreset, queue):
  renderOptions = lux.getRenderOptions()
  renderOptions.setAddToQueue(queue)
  renderOptions.setMaxSamplesRendering(samples)
  return renderOptions

def getCameras(allCameras):
  if allCameras:
    cameraToRender = lux.getCameras()[0:-1]
  else:
    cameraToRender = [lux.getCamera()]
  return cameraToRender

def aspectRatio(sceneInfo, imageWidth):
  sceneWidth = sceneInfo.get('width')
  sceneHeight = sceneInfo.get('height')
  ratioHeightWidth = sceneHeight / sceneWidth
  imageHeight = int(imageWidth * ratioHeightWidth)
  return imageHeight

def renderImage(cameraToRender, outputFolder, file, style, outputExtension, imageWidth, imageHeight, renderOptions):
  for camera in cameraToRender:
    lux.setCamera(camera)
    outputPath = outputFolder + os.path.sep + file
    outputPath = os.path.splitext(outputPath)[0] + "_" + style + "_" + str(camera) + "." + str(outputExtension)
    print("Rendering {}".format(outputPath))
    if not lux.renderImage(path = outputPath, width = imageWidth, height = imageHeight, opts = renderOptions):
      print("Rendering was cancelled by user.")
      break

def main():
  sceneInfo = lux.getSceneInfo()
  imageFormatCombo = ["jpeg", "png", "tiff", "exr"]
  lightingPresetCombo = lux.getLightingPresets()
  lightingPresetCombo.append('Currently used')
  values = [("ifold", lux.DIALOG_FOLDER, "Folder to import STEP models from:", None),
            (lux.DIALOG_LABEL, "--"),
            ("oext", lux.DIALOG_ITEM, "Output image format:", "png", imageFormatCombo),
            ("ofold", lux.DIALOG_FOLDER, "Output folder:", None),
            ("toon", lux.DIALOG_CHECK, "Include Toon pass", True),
            (lux.DIALOG_LABEL, "--"),
            (lux.DIALOG_LABEL, "Render settings:"),
            ("preset", lux.DIALOG_ITEM, "Lighting preset to use:", lightingPresetCombo[-1], lightingPresetCombo),
            ("camera", lux.DIALOG_CHECK, "Render all cameras", True),
            ("width", lux.DIALOG_INTEGER, "Render width:", sceneInfo.get('width'), (1, 10000)),
            ("height", lux.DIALOG_INTEGER, "Render height:", sceneInfo.get('height'), (1, 10000)),
            ("ratio", lux.DIALOG_CHECK, "Keep realtime aspect ratio (Use only width)", True),
            ("samples", lux.DIALOG_INTEGER, "Maximum samples:", 64),
            (lux.DIALOG_LABEL, "--"),
            (lux.DIALOG_LABEL, "Options:"),
            ("queue", lux.DIALOG_CHECK, "Add to queue", True),
            ("process", lux.DIALOG_CHECK, "Process queue after running script", False)]
  opts = lux.getInputDialog(title = "Batch Render Model Collection",
                            desc = "Renders a collection of models in a folder to images.",
                            values = values,
                            id = "batchrenderSTEPmodels.py.luxion")
  if not opts: return

  importExtension = "step"

  importFolder = opts["ifold"]
  outputExtension = opts["oext"][1]
  outputFolder = opts["ofold"]
  toonPass = opts["toon"]
  lightingPreset = opts["preset"][1]
  allCameras = opts["camera"]
  imageWidth = opts["width"]
  imageHeight = opts["height"]
  realtimeRatio = opts["ratio"]
  samples = opts["samples"]
  queue = opts["queue"]
  process = opts["process"]

  importOptions = getGeometryImportOptions()
  renderOptions = setRenderOptions(samples, lightingPreset, queue)
  cameraToRender = getCameras(allCameras)

  if realtimeRatio:
    imageHeight = aspectRatio(sceneInfo, imageWidth)

  modelList = []
  for file in os.listdir(importFolder):
    if file.endswith(importExtension):
      modelList.append(file)
      print(modelList)

      scenePath = sceneInfo.get('file')
      lux.openFile(scenePath)

      if lightingPreset != 'Currently used':
        lux.setLightingPreset(lightingPreset)

      importPath = importFolder + os.path.sep + file

      print("Importing {}".format(importPath))
      if not lux.importFile(path = importPath, opts = importOptions):
        raise Exception("Error in import! User cancelled?")

      lux.setMaterialTemplate(name = "Toy Train - Realistic")
      style = "Realistic"
      renderImage(cameraToRender, outputFolder, file, style, outputExtension, imageWidth, imageHeight, renderOptions)

      if toonPass:
        lux.setMaterialTemplate(name = "Toy Train - Toon")
        style = "Toon"
        renderImage(cameraToRender, outputFolder, file, style, outputExtension, imageWidth, imageHeight, renderOptions)

  if process:
    print("Processing queue")
    lux.processQueue()

main()